
\ Callist2.f 
\ Каллисто-2, черновик 11
\ Авторское право (c) Васильев Илья Владимирович, 15 марта 2017, Москва
\ Перед вами язык программирования для "Электроники МК-161", развивающий идеи Каллисто Классик

\ Каллисто-2 распространяется под свободной лицензией GNU GPL версии 3 или более поздней.
\ Текст лицензии GPL на русском и английском языках располагается в архиве дистрибутива в каталоге gpl.
\ Каллисто-2, а также основаные на ней работы, запрещается распространять без исходного текста и текста лицензии.
\ Распространение приложений, написанных на Каллисто-2, и использование примеров кода на Каллисто-2,
\ размещённых в Руководстве, разрешается без ограничений.
\
\ Если вы несогласны с правами, которые предоставляет свободное программное обеспечение
\ и обязанностями, которые оно налагает, удалите Каллисто-2 со своих носителей.

\ Исходный текст предназначен для SP-Forth 4.20

\ == Регистры МК-161 ==
\ R0           Временный регистр обработчика
\ R2 RP        Указатель целочисленного стека возвратов, растёт вниз от 5090
\ R3 SP        Указатель десятичного стека: растёт вниз от 998
\ R6 RI, IP    Указатель инструкций, указывает на токен в шитом коде
\ R7 W, WP     Указатель слов Форта, содержит CFA исполняемого слова
\              временный регистр обработчика
\ R8 JP        Указатель/регистр передачи управления NEXT,
\              временный регистр обработчика
\ R9 =NEXT     Адрес точки NEXT
\ RC TOS       Сохраняем вершину стека данных (когда не в RX)
\
\ R9042 =9500  Индексный регистр памяти программ
\              Содержит адрес таблицы свёрток


\ Для отладки
: .s ( -- )  ." Stack: " DEPTH IF DEPTH 00 DO I PICK . LOOP ELSE ." empty" THEN CR ;  \ Печать инструментального стека
: .HH ( b -- )  BASE @ SWAP HEX 00 <# # # #> TYPE SPACE BASE ! ;
: .8 ( a -- ) 08 00 DO DUP C@ .HH 1+ LOOP DROP ;

\ --------------------------------------
\ Области памяти МК-161 и их сохранение.
\ --------------------------------------

ONLY FORTH DEFINITIONS
CREATE prgm  10000 ALLOT         \ Память программ
prgm 10000 255 FILL              \ Заполняем FF

: .prg ( a -- ) prgm + DUP .8 08 + .8 ;

CREATE regb  8168 ALLOT          \ Регистровая память (байты)
regb 8168 ERASE                  \ Обнуляем

CREATE regd  8000 ALLOT          \ Регистровая память (десятичные регистры)
regd 8000 ERASE                  \ Обнуляем

: save.mkp ( -- )                \ Сохранить память программ в файл Callist2.mkp
  HERE 00 C,                     \ Номер начальной страницы
  prgm 10000 00 DO DUP C@ C, 01 C, 1+ LOOP DROP     \ Создать образ
  S" Callist2.mkp" W/O ( BIN )
  CREATE-FILE ABORT" Can't create MKP"
  SWAP 20001 02 PICK
  WRITE-FILE  ABORT" Can't write MKP"
  CLOSE-FILE  ABORT" Can't close MKP"
  -20001 ALLOT                                      \ Убрать образ из памяти
;

: save.mkb ( -- )                \ Сохранить байтовые регистры в файл Callist2.mkb
  S" Callist2.mkb" W/O ( BIN )
  CREATE-FILE ABORT" Can't create MKB"
  1000 regb + 4096 02 PICK
  WRITE-FILE  ABORT" Can't write MKB"
  CLOSE-FILE  ABORT" Can't close MKB"
;

: save.mkd ( -- )                \ Сохранить десятичные регистры в файл Callist2.mkd
  S" Callist2.mkd" W/O ( BIN )
  CREATE-FILE ABORT" Can't create MKD"
  regd 8000 02 PICK
  WRITE-FILE  ABORT" Can't write MKD"
  CLOSE-FILE  ABORT" Can't close MKD"
;

\ ----------------------
\ Вспомогательные слова.
\ ----------------------
: 4*  ( n1 -- n2 )  2* 2* ;                           \ Умножение на 4
: 8*  ( n1 -- n2 )  2* 2* 2* ;                        \ Умножение на 8
: 16* ( n1 -- n2 )  8* 2* ;                           \ Умножение на 16
: [+] + ;  : [-] - ;  : [1-] 1- ;  : [1+] 1+ ;        \ Сохраним операции с инструментальным стеком данных, как [+] и [-]

: WBin@ ( адрес -- n )                                \ Считать 2-х байтовое целое без знака в двоичном формате ЭКВМ
    DUP C@ 16* 16* SWAP 1+ C@ [+] ;
: WBin! ( n адрес -- )  OVER 256 / OVER C!            \ Записать 2-х байтовое целое без знака в двоичном формате ЭКВМ
    1+ SWAP 256 MOD SWAP C! ;
: WZero! ( адрес -- ) 00 OVER C!  1+ 00 SWAP C! ;     \ Обнулить два байта
: Byte>BCD ( n1 -- n2 ) 10 /MOD 16* [+] ;
: BCD>Byte ( n1 -- n2 ) 16 /MOD 10 * [+] ;
: WBCD@ ( адрес -- n )
    DUP C@ BCD>Byte 100 *  SWAP 1+ C@ BCD>Byte [+] ;  \ Считать 2-х байтовое целое число без знака в bcd-формате ЭКВМ
: WBCD! ( n адрес -- ) OVER 100 / Byte>BCD OVER C!    \ Записать 2-х байтовое целое число без знака в bcd-формате ЭКВМ
    1+ SWAP 100 MOD Byte>BCD SWAP C! ;

: getnumber ( "число" -- n )                          \ Считать следующее слово из входного потока, как число
    00 00 BL WORD COUNT >NUMBER 2DROP DROP ;

: ShiftDigits ( a -- )                                \ Сдвинуть мантиссу вправо, освободив место для новой цифры слева
  07 OVER [+] DUP C@ [1+] SWAP C!                     \ Увеличить порядок на 1
  00                                                  \ Вставить цифру 0
  06 00 DO                                            \ Цикл выполнится 6 раз
  ( a цифра )
    16* SWAP [1+] SWAP  ( a+1 16*цифра )
    OVER C@ 16 /MOD     ( a+1 16*цифра low high )
    ROT [+]             ( a+1 low newbyte )
    02 PICK C!          ( a+1 low )
  LOOP
  2DROP ;

: StoreDigit ( a цифра -- )                         \ Добавить старшую десятичную цифру к числу во внутреннем формате МК-161
    16* SWAP [1+] DUP C@  ( d*16 a+1 byte )
    ROT [+] SWAP C! ;

: dec! ( n a -- )                                   \ Записать целое n во внутреннем формате МК-161
  DUP 08 ERASE                                      \ Обнулим 8-байтный буфер
  SWAP DUP 0< IF 0x80 02 PICK C! NEGATE THEN        \ Запишем знак числа
  ( a n )
  BEGIN 10 /MOD        ( a цифра n' )
    SWAP 02 PICK SWAP  ( a n' a цифра )
    StoreDigit         ( a n' )
    DUP
  WHILE OVER ShiftDigits
  REPEAT
  2DROP ;

\ ------------------------------------
\ Метки для ссылок вперёд. Объявления.
\ ------------------------------------
01 VALUE lc                           \ Счётчик меток начинается с метки 1
00 VALUE LabelLast                    \ Метка, определяющая контекст
50 CONSTANT LBLMAX                    \ Максимальное число локальных меток
CREATE LTABLE  LBLMAX 1+ 2*  ALLOT
50 CONSTANT FRMAX                     \ Максимальное число ссылок вперёд
CREATE FRTABLE  FRMAX 4* ALLOT        \ ByteBin метка,атрибут, WBCD адрес-вхождения

LTABLE LBLMAX 1+ 2* ERASE             \ FRCLR: Инициализация таблиц ссылок вперёд
FRTABLE FRMAX 4* ERASE

: LabelClear ( -- ) 00 TO LabelLast ; \ Очистка контекста метки

: FRCHK                               \ Проверка неразрешённых ссылок вперёд
    FRMAX 00 DO I 4* FRTABLE [+] C@ ABORT" FRCHK: Unresolved forward" LOOP ;


\ ---------------------------------------------------------
\ X-компилятор исходного текста Каллисто-2 в память МК-161.
\ ---------------------------------------------------------
1000 VALUE bc                                       \ Номер текущего байтового регистра
: bhere ( -- адрес ) bc regb [+] ;                  \ Инструментальный адрес текущего байтового регистра
: ballot ( n -- ) bc [+] TO bc ;                    \ Зарезервировать n байтовых регистров
: b, ( c -- ) bhere C! 01 ballot ;                  \ Скомпилировать байт c в байтовую память
: bxt, ( xt -- ) 100 /MOD b, b, ;                   \ Скомпилировать выполнимый токен xt в байтовую память
: .borg ( "число" -- ) getnumber TO bc ;            \ Сделать байтовый регистр aaaa текущим
: b: ( "имя" -- ) CREATE bc , DOES> ( -- bc ) @ ;   \ Определить метку в байтовой памяти
: breg ( "имя" -- ) b: 01 ballot ;                  \ Дать имя очередному байтовому регистру

16 VALUE dc                                         \ Распределение десятичных регистров начинается с R16
: d! ( n a -- ) 8* regd [+] dec!  LabelClear ;      \ Присвоить целое значение десятичному регистру
: dallot ( n -- ) dc [+] TO dc ;                    \ Зарезервировать n десятичных регистров
: d, ( n -- ) dc d! 01 dallot ;                     \ Скомпилировать n на вершину десятичного словаря
: .dorg ( "число" -- ) getnumber TO dc ;            \ Сделать десятичный регистр aaaa текущим
: d: ( "имя" -- ) CREATE dc , DOES> ( -- dc ) @ ;   \ Определить метку в десятичной памяти
: dreg ( "имя" -- ) d: 01 dallot ;                  \ Дать имя очередному десятичному регистру
: dconst ( "имя" -- ) d: d, ;                       \ Дать имя и значение очередному десятичному регистру

00 VALUE pc                                         \ Счётчик команд
: p! ( b a -- )  prgm [+] C! ;                      \ Запись шага b по адресу a в память программ
: phere ( -- адрес )  pc prgm [+] ;                 \ Инструментальный адрес текущего места в памяти программ
: pallot ( n -- )  pc [+] TO pc ;                   \ Зарезервировать n шагов в памяти программ
: p, ( c -- )  phere C! 01 pallot ;                 \ Скомпилировать шаг c в память программ
: pbcd, ( n -- ) 100 MOD Byte>BCD p, ;              \ Скомпилировать в шаг младшую половину n, как число bcd
: pbcd2, ( n -- ) phere WBCD! 02 pallot ;           \ Скомпилировать в два шага bcd-число n
: pbcd!  ( n a -- ) prgm [+]
    SWAP 100 MOD Byte>BCD SWAP C! ;
: pbcd2! ( n a -- ) prgm [+] WBCD! ;                \ Записать n в два шага по адресу a, как число bcd
: ptext" ( "text"" -- )                             \ Скомпилировать строку литер в память программ
    34 WORD COUNT SWAP phere 02 PICK MOVE pallot ;
: p: ( "имя" -- )  CREATE pc , DOES> ( -- pc ) @ ;  \ Определить метку в памяти программ
: .org ( "число" -- )  getnumber TO pc ;            \ Сделать шаг по адресу aaaa текущим

00 VALUE sc                                         \ Номер свободной свёртки
: ps, ( pc -- ) 9500 prgm [+]                       \ Добавить адрес обработчика pc в таблицу свёрток
   sc 2* [+]  SWAP 256 /MOD
   02 PICK C!  SWAP [1+] C!  sc [1+] TO sc ;


\ -------------------------------------------------
\ X-компилятор входного языка "Электроники МК-161".
\ -------------------------------------------------
: 1mi ( cop "мнемоника" -- )  CREATE C, DOES> ( -- ) C@ p, ;            \ Одношаговая команда
: 2mi ( cop "мнемоника" -- )  CREATE C, DOES> ( n -- ) C@ p,  pbcd, ;   \ Двухшаговая команда

VOCABULARY Assembler

: start-code ( -- ) ALSO Assembler ;                \ Начать компилировать код на ЯМК
: end-code ( -- )  ONLY FORTH DEFINITIONS ;         \ Завершить x-определение примитива

: code ( "имя" -- )                                 \ Начать x-определение примитива
    CREATE bc ,  sc b,  pc ps,  start-code  DOES> ( -- ) @ bxt, ;

: :: ( "имя" -- )                                   \ Начать x-определение высокого уровня
    CREATE bc ,  00 b,  DOES> ( -- ) @ bxt, ;       \ Сделать ;- для оптимизации хвоста
: ;; ( -- )  ;                                      \ Может компилировать exit, но пока пишем exit вручную

: ?pairs ( n1 n2 -- )                               \ Всё ли в порядке с операторными скобками?
    - ABORT" Conditionals aren't paired" ;
: ?far ( цель откуда -- флаг )                      \ Проверить необходимость межстраничного перехода
    1+ 100 /  SWAP 100 /  = 0= ;
: ?reg61 ( регистр -- флаг )  16 < ;                \ Проверить, идёт ли обращение к регистрам МК-61
: ?regPP ( регистр -- флаг )  99 > ;                \ Проверить, идёт ли обращение к регистрам выше R99


\ ---------------------------------------
\ Умные переходы и обращения к регистрам.
\ ---------------------------------------
start-code DEFINITIONS
 07 CONSTANT x≠0                                    \ Коды условий для структур управления
 09 CONSTANT x≥0
0xC CONSTANT x<0
0xE CONSTANT x=0

\ Умные переходы
: БП   ( адрес -- )  DUP pc ?far IF 0xF1 p, pbcd2, ELSE 0x51 p, pbcd, THEN LabelClear ;
: ПП   ( адрес -- )  DUP pc ?far IF 0xF3 p, pbcd2, ELSE 0x53 p, pbcd, THEN LabelClear ;
: Fx≠0 ( адрес -- )  DUP pc ?far IF 0xF7 p, pbcd2, ELSE 0x57 p, pbcd, THEN LabelClear ;
: FL2  ( адрес -- )  DUP pc ?far IF 0xF8 p, pbcd2, ELSE 0x58 p, pbcd, THEN LabelClear ;
: Fx≥0 ( адрес -- )  DUP pc ?far IF 0xF9 p, pbcd2, ELSE 0x59 p, pbcd, THEN LabelClear ;
: FL3  ( адрес -- )  DUP pc ?far IF 0xFA p, pbcd2, ELSE 0x5A p, pbcd, THEN LabelClear ;
: FL1  ( адрес -- )  DUP pc ?far IF 0xFB p, pbcd2, ELSE 0x5B p, pbcd, THEN LabelClear ;
: Fx<0 ( адрес -- )  DUP pc ?far IF 0xFC p, pbcd2, ELSE 0x5C p, pbcd, THEN LabelClear ;
: FL0  ( адрес -- )  DUP pc ?far IF 0xFD p, pbcd2, ELSE 0x5D p, pbcd, THEN LabelClear ;
: Fx=0 ( адрес -- )  DUP pc ?far IF 0xFE p, pbcd2, ELSE 0x5E p, pbcd, THEN LabelClear ;

\ Структуры управления, можно упростить реализацию с помощью умных переходов
: begin ( -- цель 101 )  pc 101 ;                   \ Отметить, куда будет переход назад, начало цикла
: until ( цель 101 условие -- ) SWAP 101 ?pairs     \ Условный переход назад, цикл "до"
    OVER pc ?far IF 0xF0 [+] p, pbcd2, ELSE 0x50 [+] p, pbcd, THEN ;
: again ( цель 101 -- ) 101 ?pairs                  \ Безусловный переход назад, бесконечный цикл
    DUP pc ?far IF 0xF1 p, pbcd2, ELSE 0x51 p, pbcd, THEN ;
: fl0 ( цель 101 -- ) 101 ?pairs                    \ Цикл по R0 назад, begin..fl0 = FL0/PFL0
    DUP pc ?far IF 0xFD p, pbcd2, ELSE 0x5D p, pbcd, THEN ;
: fl1 ( цель 101 -- ) 101 ?pairs                    \ Цикл по R1 назад, begin..fl1 = FL1/PFL1
    DUP pc ?far IF 0xFB p, pbcd2, ELSE 0x5B p, pbcd, THEN ;
: fl2 ( цель 101 -- ) 101 ?pairs                    \ Цикл по R2 назад, begin..fl2 = FL2/PFL2
    DUP pc ?far IF 0xF8 p, pbcd2, ELSE 0x58 p, pbcd, THEN ;
: fl3 ( цель 101 -- ) 101 ?pairs                    \ Цикл по R3 назад, begin..fl3 = FL3/PFL3
    DUP pc ?far IF 0xFA p, pbcd2, ELSE 0x5A p, pbcd, THEN ;

: if ( условие -- откуда 102 )  0xF0 [+] p,         \ Условный переход вперёд
    pc 00 p, 00 p, 102 ;                            \ (можно добавить ifs для внутристраничного перехода)
: then ( откуда 102 -- )  102 ?pairs                \ Конец условного перехода вперёд
    pc SWAP pbcd2! ;

\ Умный доступ к регистрам МК-161
: П ( регистр -- )                                  \ Запись в произвольный регистр МК-161
    DUP ?reg61 IF 0x40 [+] p,
    ELSE DUP ?regPP IF 0xF4 p, pbcd2, ELSE 0x4F p, pbcd, THEN
    THEN
;

: ИП ( регистр -- )                                 \ Чтение произвольного регистра МК-161
    DUP ?reg61 IF 0x60 [+] p,
    ELSE DUP ?regPP IF 0xF6 p, pbcd2, ELSE 0x6F p, pbcd, THEN
    THEN
;

: pnum, ( число -- ) -1 SWAP                        \ Скомпилировать целое число в команды языка МК-161
    BEGIN 10 /MOD DUP 0= UNTIL DROP                 \ Сперва кладём все цифры в стек
    BEGIN p, DUP 0< UNTIL DROP                      \ Потом компилируем их в нужном порядке
    LabelClear
;

\ ------------------------------------
\ Метки для ссылок вперёд. Реализация.
\ ------------------------------------
start-code DEFINITIONS
: FRRES4 ( адрес -- )                 \ Атрибут 04: четырёхшаговый ввод числа, можно добавить проверку, достаточно ли 3 шагов
    pc SWAP TO pc
    DUP  1000 /MOD p,  100 /MOD p,  10 /MOD p, p,  TO pc ;

: FRRES3 ( адрес -- )                 \ Атрибут 03: трёхшаговый ввод числа
    pc 999 > ABORT" FRRES3: Constant is too big"
    pc SWAP TO pc
    DUP  100 /MOD p,  10 /MOD p, p,  TO pc ;

: FRRES2 ( адрес -- )                 \ Атрибут 02: трёхшаговый переход вроде РБП, можно добавить проверку на внутристраничный переход
    pc SWAP 1+ pbcd2! ;

: FRRES1 ( адрес -- )                 \ Атрибут 01: двухшаговый переход вроде БП
    pc OVER ?far ABORT" FRRES1: Forward reference is out of page" 
    pc SWAP 1+ pbcd! ;
    
: FRRES  ( -- )                       \ Разрешение контекстной ссылки вперёд
    FRMAX 00 DO  I 4* FRTABLE [+] DUP C@ LabelLast =
      IF ( вхождение )
        DUP 1+ C@                     \ Считываем атрибут
        OVER WZero!                   \ Обнуляем поля атрибута и метки
        SWAP 2+ WBCD@ SWAP            \ Получаем адрес ссылки вперёд
        DUP 04 = IF DROP FRRES4 ELSE  \ Вызываем нужный обработчик по атрибуту
        DUP 03 = IF DROP FRRES3 ELSE
        DUP 02 = IF DROP FRRES2 ELSE
        01 = IF FRRES1 THEN THEN THEN THEN
      ELSE DROP THEN
    LOOP ;

: FRADD  ( атрибут -- )               \ Добавление контекстной ссылки вперёд в таблицу
    FRMAX 1+ 00
    DO FRMAX I = ABORT" FRADD: Too many forward references"
      I 4* FRTABLE [+] DUP C@ 0=
      IF LabelLast OVER C!            \ Записываем метку
         pc OVER 2+ WBCD!             \ Записываем адрес
         1+ C!                        \ Записываем атрибут
         LabelClear LEAVE ELSE DROP THEN
    LOOP ;

: Label ( "метка" -- )                \ Определить метку
    CREATE  lc DUP ,  1+ DUP TO lc  LBLMAX > ABORT" Label: Too many labels"
    DOES> ( -- значение | 0 )  @ DUP TO LabelLast  2* LTABLE + WBCD@ ;

: &: ( 0 -- )                         \ Установить метку на текущее место в памяти программ МК-161
    ABORT" &:: Duplicate definition"
    FRRES  pc  LabelLast 2* LTABLE + WBCD!  LabelClear ;

end-code ALSO Assembler

: &! ( n 0 -- )
    ABORT" &=: Duplicate definition"
    pc SWAP TO pc
    FRRES pc LabelLast 2* LTABLE + WBCD!  LabelClear  TO pc ;

start-code DEFINITIONS

\ Возможно слияние реализации с "умными" командами для случая перехода назад
\ Ещё можно продумать ссылку на регистры, определяемые позднее
\ Также надо продумать смещения, включая отрицательные

: &2mi ( cop "мнемоника" -- )  CREATE C, DOES> ( a -- ) 01 FRADD  C@ p,  pbcd, ;   \ Двухшаговая команда со ссылкой вперёд
: &3mi ( cop "мнемоника" -- )  CREATE C, DOES> ( a -- ) 02 FRADD  C@ p,  pbcd2, ;  \ Трёхшаговая команда со ссылкой вперёд

: &&&,  ( n -- )
    DUP 999 > ABORT" &&&,: Constant is too big"
    03 FRADD  100 /MOD p,  10 /MOD p, p, ;
: &&&&,  ( n -- )
    04 FRADD 1000 /MOD p,  100 /MOD p,  10 /MOD p, p, ; \ Можно добавить проверку, достаточно ли 3 шагов

\ -----------------------
\ Таблица кодов операций.
\ -----------------------
   0 1mi 0         1 1mi 1          2 1mi 2          3 1mi 3         4 1mi 4         5 1mi 5         6 1mi 6        7 1mi 7
   8 1mi 8         9 1mi 9        0xA 1mi ,        0xB 1mi /-/     0xC 1mi ВП      0xD 1mi Cx      0xE 1mi В↑     0xF 1mi FВx
0x10 1mi +      0x11 1mi -       0x12 1mi ×       0x13 1mi ÷      0x14 1mi ↔      0x15 1mi F10ˣ   0x16 1mi Feˣ   0x17 1mi Flg
0x18 1mi Fln    0x19 1mi Farcsin 0x1A 1mi Farccos 0x1B 1mi Farctg 0x1C 1mi Fsin   0x1D 1mi Fcos   0x1E 1mi Ftg
0x20 1mi Fπ     0x21 1mi F√      0x22 1mi Fx²     0x23 1mi F1/x   0x24 1mi Fxʸ    0x25 1mi Fo     0x26 1mi КМ→Г  0x27 1mi K-
0x28 1mi КИПРГ  0x29 1mi К÷      0x2A 1mi КМС→Г
0x30 1mi КГ→МС  0x31 1mi К|x|    0x32 1mi КЗН     0x33 1mi КГ→М   0x34 1mi К[x]   0x35 1mi К{x}   0x36 1mi Кmax  0x37 1mi КAND
0x38 1mi КOR    0x39 1mi КXOR    0x3A 1mi КNOT    0x3B 1mi КСЧ                                   
0x40 1mi П0     0x41 1mi П1      0x42 1mi П2      0x43 1mi П3     0x44 1mi П4     0x45 1mi П5     0x46 1mi П6    0x47 1mi П7
0x48 1mi П8     0x49 1mi П9      0x4A 1mi ПА      0x4B 1mi ПВ     0x4C 1mi ПС     0x4D 1mi ПД     0x4E 1mi ПЕ
0x50 1mi С/П    0x51 &2mi &БП    0x52 1mi В/О     0x53 &2mi &ПП   0x54 1mi КНОП   0x55 1mi КЭКР   0x56 1mi КГРФ  0x57 &2mi &Fx≠0
0x58 &2mi &FL2  0x59 &2mi &Fx≥0  0x5A &2mi &FL3   0x5B &2mi &FL1  0x5C &2mi &Fx<0 0x5D &2mi &FL0  0x5E &2mi &Fx=0
0x60 1mi ИП0    0x61 1mi ИП1     0x62 1mi ИП2     0x63 1mi ИП3    0x64 1mi ИП4    0x65 1mi ИП5    0x66 1mi ИП6   0x67 1mi ИП7
0x68 1mi ИП8    0x69 1mi ИП9     0x6A 1mi ИПА     0x6B 1mi ИПВ    0x6C 1mi ИПС    0x6D 1mi ИПД    0x6E 1mi ИПЕ
0x70 1mi Кx≠00  0x71 1mi Кx≠01   0x72 1mi Кx≠02   0x73 1mi Кx≠03  0x74 1mi Кx≠04  0x75 1mi Кx≠05  0x76 1mi Кx≠06 0x77 1mi Кx≠07
0x78 1mi Кx≠08  0x79 1mi Кx≠09   0x7A 1mi Кx≠0А   0x7B 1mi Кx≠0В  0x7C 1mi Кx≠0С  0x7D 1mi Кx≠0Д  0x7E 1mi Кx≠0Е 0x7F 2mi РКx≠0
0x80 1mi КБП0   0x81 1mi КБП1    0x82 1mi КБП2    0x83 1mi КБП3   0x84 1mi КБП4   0x85 1mi КБП5   0x86 1mi КБП6  0x87 1mi КБП7
0x88 1mi КБП8   0x89 1mi КБП9    0x8A 1mi КБПА    0x8B 1mi КБПВ   0x8C 1mi КБПС   0x8D 1mi КБПД   0x8E 1mi КБПЕ  0x8F 2mi РКБП
0x90 1mi Кx≥00  0x91 1mi Кx≥01   0x92 1mi Кx≥02   0x93 1mi Кx≥03  0x94 1mi Кx≥04  0x95 1mi Кx≥05  0x96 1mi Кx≥06 0x97 1mi Кx≥07
0x98 1mi Кx≥08  0x99 1mi Кx≥09   0x9A 1mi Кx≥0А   0x9B 1mi Кx≥0В  0x9C 1mi Кx≥0С  0x9D 1mi Кx≥0Д  0x9E 1mi Кx≥0Е 0x9F 2mi РКx≥0
0xA0 1mi КПП0   0xA1 1mi КПП1    0xA2 1mi КПП2    0xA3 1mi КПП3   0xA4 1mi КПП4   0xA5 1mi КПП5   0xA6 1mi КПП6  0xA7 1mi КПП7
0xA8 1mi КПП8   0xA9 1mi КПП9    0xAA 1mi КППА    0xAB 1mi КППВ   0xAC 1mi КППС   0xAD 1mi КППД   0xAE 1mi КППЕ  0xAF 2mi РКПП
0xB0 1mi КП0    0xB1 1mi КП1     0xB2 1mi КП2     0xB3 1mi КП3    0xB4 1mi КП4    0xB5 1mi КП5    0xB6 1mi КП6   0xB7 1mi КП7
0xB8 1mi КП8    0xB9 1mi КП9     0xBA 1mi КПА     0xBB 1mi КПВ    0xBC 1mi КПС    0xBD 1mi КПД    0xBE 1mi КПЕ   0xBF 2mi РКП
0xC0 1mi Кx<00  0xC1 1mi Кx<01   0xC2 1mi Кx<02   0xC3 1mi Кx<03  0xC4 1mi Кx<04  0xC5 1mi Кx<05  0xC6 1mi Кx<06 0xC7 1mi Кx<07
0xC8 1mi Кx<08  0xC9 1mi Кx<09   0xCA 1mi Кx<0А   0xCB 1mi Кx<0В  0xCC 1mi Кx<0С  0xCD 1mi Кx<0Д  0xCE 1mi Кx<0Е 0xCF 2mi РКx<0
0xD0 1mi КИП0   0xD1 1mi КИП1    0xD2 1mi КИП2    0xD3 1mi КИП3   0xD4 1mi КИП4   0xD5 1mi КИП5   0xD6 1mi КИП6  0xD7 1mi КИП7
0xD8 1mi КИП8   0xD9 1mi КИП9    0xDA 1mi КИПА    0xDB 1mi КИПВ   0xDC 1mi КИПС   0xDD 1mi КИПД   0xDE 1mi КИПЕ  0xDF 2mi РКИП
0xE0 1mi Кx=00  0xE1 1mi Кx=01   0xE2 1mi Кx=02   0xE3 1mi Кx=03  0xE4 1mi Кx=04  0xE5 1mi Кx=05  0xE6 1mi Кx=06 0xE7 1mi Кx=07
0xE8 1mi Кx=08  0xE9 1mi Кx=09   0xEA 1mi Кx=0А   0xEB 1mi Кx=0В  0xEC 1mi Кx=0С  0xED 1mi Кx=0Д  0xEE 1mi Кx=0Е 0xEF 2mi РКx=0
                0xF1 &3mi &РБП   0xF2 1mi РРВ/О   0xF3 &3mi &РПП                                                 0xF7 &3mi &Px≠0
0xF8 &3mi &PFL2 0xF9 &3mi &Рx≥0  0xFA &3mi &PFL3  0xFB &3mi &PFL1 0xFC &3mi &Px<0 0xFD &3mi &PFL0 0xFE &3mi &Px=0

: next; ( -- )  КБП9 end-code ;
: БП;  ( a -- )  БП end-code ;

end-code

\ -------------------------------------
\ Каллисто-2: Байтовые регистры МК-161.
\ -------------------------------------
b: bufKbd  08 ballot  \ Буфер клавиатуры размером в 8 байт
breg kbdHead          \ Голова буфера клавиатуры: 0..7
breg kbdTail          \ Хвост буфера клавиатуры: 0..7
breg bKey             \ Код нажатой клавиши (или 0)
breg flgKbdCaps       \ Флаг клавиатуры: строчные / заглавные = 0 / 1
breg bKbdCodeFK       \ Предыдущая цифра при FK xx

\ ---------------------------------------
\ Каллисто-2: Десятичные регистры МК-161.
\ ---------------------------------------
dreg ptrKbdInt        \ Адрес активного режима клавиатуры

\ -----------------------------------
\ Каллисто-2: Память программ МК-161.
\ -----------------------------------
start-code
Label &Autoexec
Cx                           \ === Запускаем шитый код ===
9 5 0 0                      \ Адрес таблицы свёрток
9042 П                       \ Индексный регистр памяти программ
5 0 9 0 П2                   \ Инициализация стека возвратов
9 9 8 П3                     \ Инициализация стека данных
&Autoexec &&&&, П6           \ RI := Начало шитого кода - 1
КБП9                         \ R9 содержит адрес точки NEXT

p: CALL                      \ Обработчик слов высокого уровня
ИП6 В↑ ВП 2 /-/ К[x] КП2     \ Сохраняем старшую половину RI
ВП 2 - КП2 Fo                \ Сохраняем младшую половину RI
ИП7 П6 Fo                    \ RI := PFA - 1 = CFA
                             \ Для быстродействия CALL склеено с NEXT

p: NEXT                      \ Адресный интерпретатор: выполнить следующий токен в шитом коде
КИП6 ВП 2 КИП6 + П7 Fo       \ W := REGW[RI++], считываем CFA в R7
КИП7 9210 П                  \ Находим по CFA адрес обработчика
П8 Fo КБП8                   \ Передаём на него управление
\ 1. Можно сделать если первый КИП6 даёт ноль, то второй КИП6 даёт сразу свёртку -- без CFA
\ такой подход сэкономит сотню-другую байт в памяти данных, но чуть замедлит Каллисто
\ и запутает компилятор
\ 2. Второй точкой входа может быть команда Fo перед NEXT

CALL ps,                     \ Обработчик 000: CALL

\ -------------------
\ Драйвер клавиатуры.
\ -------------------
begin
p: ChPutBell                 \ Издать короткий звуковой сигнал
  4 4 0 В↑ 9
  9052 П
  РРВ/О                      \ Может завершать прерывание клавиатуры или таймера

p: Timer1Int                 \ Прерывание: таймер 1
  2 0 9051 П                 \ 200 мс, 5 раз/секунду

p: KeyAdd
  kbdHead ИП                 \ Голова очереди клавиатуры
  kbdTail ИП ПА 1 + 7 КAND   \ Сдвинуть хвост на 1 байт
  - x≠0                      \ Если совпадёт с головой, недовольно пискнуть
until

FВx kbdTail П                \ Сдвинуть хвост
bufKbd pnum, ИПА + ПА        \ RA: куда записать нажатую клавишу
bKey ИП КПА                  \ Записать в очередь клавиатуры код нажатой клавиши
p: Iret
РРВ/О                        \ Возврат из прерывания

p: tblKeyNum                 \ Кнопка в литеры, режим Num
ptext" 0123456789,. "        \ Кнопки 0..9 , /-/ ВП
08 p, 24 p,                  \ Кнопки Cx В↑
ptext" +-"                   \ Кнопки + -
179 p, 176 p, 183 p,         \ Кнопки × ÷ ↔
00 p, 00 p, 00 p,            \ Кнопки F K P
ptext" ():;@![]"
00 p, 27 p, 13 p,            \ Кнопки Р-ГРД-Г ВЫХОД ВВОД
06 p, 02 p, 16 p, 14 p,      \ Выбор ^f ^b ^p ^n

p: tblKeyNumF                \ Кнопка в литеры, режим Num с префиксом F
190 p,                       \ Клавиша ˣ
ptext" '" 34 p,              \ Клавиши ' "
ptext" #$%^&*~"              \ Кнопки 3..9
181 p, ptext" \" 09 p,       \ Клавиши , . пробел
127 p, ptext" |"             \ Кнопки Сx В↑
185 p, 251 p, 189 p,         \ Клавиши π √ ²
ptext" /" 191 p,             \ Кнопки ÷ ↔
00 p, 00 p, 00 p,            \ Кнопки F K P
ptext" <=" 177 p, 178 p,     \ Кнопки ШГ> ШГ< В/О С/П в < = ≥ ≠
ptext" >?{}"                 \ Кнопки ИП П БП ПП
00 p, 200 p, 192 p,          \ Кнопки Р-ГРД-Г ВЫХОД ВВОД
05 p, 01 p, 18 p, 20 p,      \ Выбор ^e ^a ^r ^t

p: tblKeyLat                 \ Кнопка в литеры, режим ЛАТ
ptext" xstunopijkyz " 08 p,  \ Кнопки 0..9 , ВП Сx
ptext" wqlrmv"               \ Кнопки В↑ + - × ÷ ↔
00 p, ptext" d" 00 p,        \ Кнопки F K P
ptext" _abcefgh"             \ Кнопки ШГ> ШГ< В/О С/П ИП П БП ПП
00 p, 27 p, 13 p,            \ Кнопки Р-ГРД-Г ВЫХОД ВВОД
06 p, 02 p, 16 p, 14 p,      \ Выбор ^f ^b ^p ^n

p: tblKeyLatF                \ Кнопка в литеры, режим ЛАТ с префиксом F
ptext" XSTUNOPIJKYZ"         \ Кнопки 0..9 ,
09 p, 127 p,                 \ Кнопки ВП Сx
ptext" WQLRMV"               \ Кнопки В↑ + - × ÷ ↔
00 p, ptext" D" 00 p,        \ Кнопки F K P
ptext" `ABCEFGH"             \ Кнопки ШГ> ШГ< В/О С/П ИП П БП ПП
00 p, 200 p, 192 p,          \ Кнопки Р-ГРД-Г ВЫХОД ВВОД
05 p, 01 p, 18 p, 20 p,      \ Выбор ^e ^a ^r ^t

p: tblKeyCtrl                \ Кнопка в литеры, режим Num с префиксом K
24 p, 19 p, 20 p, 21 p,             \ ^X ^S ^T ^U
14 p, 15 p, 16 p,                   \ ^N ^O ^P
09 p, 10 p, 11 p, 25 p, 26 p,       \ ^I ^J ^K ^Y ^Z
31 p, 28 p,                         \ K ВП и К Cx это ^_ и ^\
23 p, 17 p, 12 p, 18 p, 13 p, 22 p, \ ^W ^Q ^L ^R ^M ^V
255 p, 04 p, 127 p,                 \ K F, K K и K P это 255, ^D и 127
00 p, 01 p, 02 p, 03 p,             \ ^@ ^A ^B ^C
05 p, 06 p, 07 p, 08 p,             \ ^E ^F ^G ^H
30 p, 27 p, 29 p,                   \ K Р-ГРД-Г, К ВЫХОД и К ВВОД это ^^, ^[ и ^]
26 p, 27 p, 24 p, 25 p,             \ Выбор

p: tblKeyRus                 \ Кнопка в литеры, режим РУС
237 p, 232 p, 233 p, 234 p,  \ Клавиши э ш щ ъ
227 p, 228 p, 229 p,         \ Клавиши у ф х
174 p, 175 p, 224 p,         \ Клавиши о п р
238 p, 239 p, 32 p, 08 p,    \ Клавиши ю я пробел BS
236 p, 230 p, 225 p, 231 p,  \ Клавиши ь ц с ч
226 p, 235 p,                \ Клавиши т ы
00 p, 241 p, 00 p,           \ Кнопка F клавиша ё кнопка P
166 p, 167 p, 168 p, 169 p,  \ Клавиши ж з и й
170 p, 171 p, 172 p, 173 p,  \ Клавиши к л м н
00 p, 164 p, 165 p,          \ Кнопка Р-ГРД-Г клавиши д е
162 p, 161 p, 160 p, 163 p,  \ Клавиши в б а г

p: tblKeyRusF                \ Кнопка в литеры, режим РУС с префиксом F
157 p, 152 p, 153 p, 154 p,  \ Клавиши Э Ш Щ Ъ
147 p, 148 p, 149 p,         \ Клавиши У Ф Х
142 p, 143 p, 144 p,         \ Клавиши О П Р
158 p, 159 p, 09 p, 127 p,   \ Клавиши Ю Я ТАБ ЗБ
156 p, 150 p, 145 p, 151 p,  \ Клавиши Ь Ц С Ч
146 p, 155 p,                \ Клавиши Т Ы
00 p, 240 p, 00 p,           \ Кнопка F клавиша Ё кнопка P
134 p, 135 p, 136 p, 137 p,  \ Клавиши Ж З И Й
138 p, 139 p, 140 p, 141 p,  \ Клавиши К Л М Н
00 p, 132 p, 133 p,          \ Кнопка Р-ГРД-Г клавиши Д Е
130 p, 129 p, 128 p, 131 p,  \ Клавиши В Б А Г

Label &KPRus   Label &KPRusF \ Объявляем обработчики нажатий на клавиши
Label &KPLat   Label &KPLatF
Label &KPNum   Label &KPNumF
Label &KPCtrl  Label &KPFK

p: KeyPress                  \ Прерывание: нажатие на кнопку клавиатуры
  9029 ИП  ПА                \ RA := код нажатой кнопки
  ptrKbdInt РКБП             \ Обработать в соответствии со состоянием клавиатуры

p: KP2FK  &KPFK &&&,         \ Нажали FK в состоянии Num
p: KPStateExit
     ptrKbdInt П

p: KeyRelease                \ Прерывание: отпускание кнопки клавиатуры
  Cx bKey П                  \ Сбрасываем код нажатой клавиши
  9151 П                     \ Запретить аппаратное прерывание от таймера 1
  9051 П                     \ Обнуляем таймер 1
  РРВ/О                      \ Возврат из прерывания

p: KP2Num   &KPNum &&&,   KPStateExit БП
p: KP2NumF  &KPNumF &&&,  KPStateExit БП
p: KP2Rus   &KPRus &&&,   KPStateExit БП
p: KP2RusF  &KPRusF &&&,  KPStateExit БП
p: KP2Lat   &KPLat &&&,   KPStateExit БП
p: KP2LatF  &KPLatF &&&,  KPStateExit БП
p: KP2Ctrl  &KPCtrl &&&,  KPStateExit БП

p: KPCaps  1 flgKbdCaps ИП - flgKbdCaps П  KeyRelease БП  \ Нажали Caps в состоянии Num F

&KPCtrl &:                   \ -- Обработать клавишу в состоянии F K xx
  &KPNum &&&, ptrKbdInt П
  Cx tblKeyCtrl pnum, + КИПРГ

p: KPStore
  bKey П                   \ Записать код литеры в bKey
  5 0 9051 П               \ Задержка 500 мс (1/2 с) перед повтором
  3 9151 П                 \ Регистр разрешения аппаратных прерываний: окончание счёта таймера 1: прерывание 3
  KeyAdd БП                \ Добавить клавишу в очередь

Label &KPR1
Label &KPRF1

&KPRus  &:                   \ -- Обработать клавишу в состоянии РУС
  1 2 - &KPR1 &Fx≠0          \ Кнопка 12: Клавиша пробел
  1 - &KPR1 &Fx≠0            \ Кнопка 13: Клавиша BS
  7 - KP2RusF Fx≠0           \ Кнопка 20: Клавиша F
  2 - KP2Num Fx≠0            \ Кнопка 22: Клавиша Num
  9 - KP2Lat Fx≠0            \ Кнопка 31: Клавиша ЛАТ
  flgKbdCaps ИП  &KPRF1 &Px=0
&KPR1 &:
  tblKeyRus pnum,
p: KPFetchTbl
  ИПА + КИПРГ  KPStore Fx=0
  KeyRelease БП

&KPRusF &:                   \ -- Обработать клавишу в состоянии РУС F
  &KPRus pnum, ptrKbdInt П   \ Вернуть клавиатуру в состояние РУС
  Cx 1 2 - &KPRF1 &Px≠0      \ Кнопка 12: Клавиша ТАБ
  1 - &KPRF1 &Px≠0           \ Кнопка 13: Клавиша ЗБ
  9 - KPCaps Fx≠0            \ Кнопка 22: Клавиша Caps
  flgKbdCaps ИП  &KPR1 Fx=0
&KPRF1 &:
  tblKeyRusF pnum,  KPFetchTbl БП

Label &KPL1
Label &KPLF1

&KPLat  &:                   \ -- Обработать клавишу в состоянии ЛАТ
  1 2 - &KPL1 &Fx≠0          \ Кнопка 12: Клавиша пробел
  1 - &KPL1 &Fx≠0            \ Кнопка 13: Клавиша BS
  7 - KP2LatF Fx≠0           \ Кнопка 20: Клавиша F
  2 - KP2Num Fx≠0            \ Кнопка 22: Клавиша Num
  1 - &KPL1 &Fx≠0            \ Кнопка 23: Клавиша _
  8 - &KPL1 &Fx<0            \ Кнопки 31 и выше
  flgKbdCaps ИП  &KPLF1 &Px=0
&KPL1 &:
  tblKeyLat pnum,  KPFetchTbl БП

&KPLatF &:                   \ -- Обработать клавишу в состоянии ЛАТ F
  &KPLat pnum, ptrKbdInt П   \ Вернуть клавиатуру в состояние ЛАТ
  Cx 1 2 - &KPLF1 &Px≠0      \ Кнопка 12: Клавиша пробел
  1 - &KPLF1 &Px≠0           \ Кнопка 13: Клавиша BS
  9 - KPCaps Fx≠0            \ Кнопка 22: Клавиша Caps
  1 - &KPLF1 &Px≠0           \ Кнопка 23: Клавиша `
  8 - KP2Rus Fx≠0            \ Кнопка 31: Клавиша РУС
  &KPLF1 &Px<0               \ Кнопки 32 и выше
  flgKbdCaps ИП  &KPL1 Fx=0
&KPLF1 &:
  tblKeyLatF pnum,  KPFetchTbl БП

&KPNum &:                    \ -- Обработать клавишу в состоянии Num
  tblKeyNum pnum, + КИПРГ  KPStore Fx=0    \ Перевести код кнопки в код литеры
  ИПА 2 0 - KP2NumF Fx≠0     \ Кнопка 20: Клавиша F
  1 - KP2Ctrl Fx≠0           \ Кнопка 21: Клавиша K
  1 0 - KP2Lat Fx≠0          \ Кнопка 31: Клавиша ЛАТ
  KeyRelease БП

&KPNumF &:                   \ -- Обработать клавишу в состоянии Num F
  &KPNum pnum, ptrKbdInt П   \ Вернуть клавиатуру в состояние Num
  Cx tblKeyNumF pnum, + КИПРГ  KPStore Fx=0   \ Перевести код кнопки в код литеры
  ИПА 2 1 - KP2FK Fx≠0       \ Кнопка 21: Клавиша K
  1 - KPCaps Fx≠0            \ Кнопка 22: Клавиша Caps
  9 - KP2Rus Fx≠0            \ Кнопка 31: Клавиша РУС
  KeyRelease БП

p: KTH1  ИПА
p: KTH2  В/О
p: KTH4  1 3 + В/О
p: KeyToHex
  1 0 - KTH1 Fx≥0
  1 1 - KTH4 Fx≠0   \ Кнопка 21 это D = 13
  3 - KTH2 Fx≥0     \ Кнопки 22 и 23 не используются
  3 - KTH4 Fx≥0     \ Кнопки 24, 25 и 26 это A, B и C
  1 ↔ - KTH2 Fx≥0
  1 5 ↔ - В/О       \ Кнопки 27 и 28 это E и F

p: KPFK2
  &KPNum pnum, ptrKbdInt П
  ИПА KeyToHex ПП  KeyRelease Fx≥0
  bKbdCodeFK ИП  1 6 × + KPStore БП

&KPFK   &:                   \ -- Обработать клавишу в состоянии F K xx
  KeyToHex ПП  bKbdCodeFK П  KP2Num Fx≥0
  KPFK2 pnum,  KPStateExit БП

p: CsrOn
p: CsrOff
  9001 ИП ↔                  \ Сохраняем атрибут вывода в RX1
  3 9001 П
  8 В↑ 9013 П
  КГРФ
  FВx 9001 П В/О

p: ChGet                     \ Ждём нажатия клавиши и берём её код из очереди клавиатуры
CsrOn ПП
begin
  kbdTail ИП kbdHead ИП - x≠0
until
bufKbd pnum, FВx + ПА КИПА ПА
FВx 1 + 7 КAND kbdHead П
CsrOff ПП
ИПА В/О

p: ChPutLF                   \ Перевод строки, пока без прокрутки
9000 ИП ↔ 8 + ↔ 9000 П В/О

p: ChPut2                    \ Вывод литеры из RA на графический экран
ИПА 9020 П В/О

p: ChPut                     \ Терминальный ввод/вывод
ПА
9000 ИП
1 2 1 - ChPut2 Fx≥0
Cx 9000 П
ChPutLF ПП
ChPut2 БП

begin
  FВx - В/О
p: Sign32                    \ Преобразование 32-битного беззнакового в 32-битное со знаком.
  3 2 7 6 8  В↑  6 5 5 3 6  ×  -
  x<0
until
FВx + В/О

end-code

\ Основной код инициализации Каллисто-2
code init                    \ === Примитив init ===
  1 9148 П                   \ Сброс регистров прерываний
  KeyPress pnum, В↑ 1 2 7    \ Вектор KeyPress, приоритет 127
  9131 П                     \  → регистр настройки прерывания 1
  1 9153 П                   \ Регистр разрешения аппаратных прерываний: нажатие на кнопку клавиатуры: прерывание 1
  KeyRelease pnum, В↑ 1 2 8  \ Вектор KeyRelease, приоритет 128
  9132 П                     \  → регистр настройки прерывания 2
  2 9157 П                   \ Регистр разрешения аппаратных прерываний: отпускание кнопки клавиатуры: прерывание 2
  Timer1Int pnum, В↑ 1 2 6   \ Вектор Timer1Int, приоритет 126
  9133 П                     \  → регистр настройки прерывания 3
  1 9130 П                   \ Общее разрешение прерываний: прерывания разрешены
  2 9010 П                   \ Инициализация графического экрана
next;                        \ NEXT

code exit                    \ === Примитив exit ===
  ИП2 П8 П5  2 + П2 Fo       \ RP += 2
  КИП5 ВП 2 КИП8 + П6 Fo     \ RPOP RI
next;                        \ NEXT

start-code
  begin
    FВx + КБП9
code literal
    ↔ КП3 ↔
    КИП6 2 5 6 × КИП6 +
    3 2 7 6 8 -  x≥0
  until
  FВx -
next;

code lit8 ↔ КП3 ↔ КИП6 next;
code 2drop  p: j2drop  2  p: ndrop  ИП3 + П3 П0  КИП0 КИП0  next;
code swap  ↔ next;
\ code 2swap  ИП3 П4 П5  КИП4 ИПС КП5 ↔ ПС  КИП4 03 РКИП КП5 ↔ 03 РКП  next;
code over  ↔ КП3  next;

code ?dup  Кx≠09             \ Здесь склейка: нет NEXT
code dup  ↔ КП3  p: jdup2  ↔ В↑  next;
code pick  Кx≥09 jdup2 Fx≠0  ИП3 + ПС ↔ КИПС  next;

code 2dup  03 РКИП  КП3  ↔ КП3 ↔ next;
code rot  ↔ ↔ 03 РКИП  FВx 03 РКП  Fo next;
code drop p: jdrop  ↔  p: jdrop2  03 РКИП ↔  1 ИП3 + П3 Fo next;


code 0=  x≠0
  if  p: STF  Cx  КБП9
code 0>  /-/ STF Fx<0
  then
  p: STT  Cx 1 /-/  next;

code 0<  STT Fx≥0 Cx next;

code =  - x≠0
  if  p: STFP  Cx  jdrop2 БП
code <  - STFP Fx<0
  then
  p: STTP  1 /-/  jdrop2 БП;

code >  - STTP Fx≥0  STFP БП;

code 1+  1 +  next;
code 2+  2 +  next;
code 1-  1 -  next;
code 2-  2 -  next;
code 2×  В↑ +  next;
code 2/  2 ÷ К[x]  next;
code +  +  jdrop2 БП;
code -  -  jdrop2 БП;
code ×  ×  jdrop2 БП;
code ÷  ÷  jdrop2 БП;
code /  ÷ К[x]  jdrop2 БП;
code mod  ↔ В↑ FВx ÷ FВx ↔ К[x] × -  jdrop2 БП;
\ code /mod  03 РКИП В↑ ИПС ÷ FВx ↔ К[x] ПС × - 03 РКП  next;
\ code ×/  ИП3 П4  03 РКИП КИП4 × ИПС ÷ К[x] ПС  2 ndrop БП;
\ code ×/mod  1 ИП3 + П8  03 РКИП КИП8 × В↑ ИПС ÷ FВx ↔ К[x] ПС × - КП8  ИП8 П3  next;

code not  К[x]  Sign32 ПП  1 + /-/  next;
code min  Кmax ↔  jdrop2 БП;
code max  Кmax  jdrop2 БП;

code 10ˣ  F10ˣ  next;
code eˣ  Feˣ  next;
code lg  Flg  next;
code ln  Fln  next;
code x²  Fx²  next;
code √  F√   next;
code 1/x  F1/x  next;
code yˣ  ↔ Fxʸ  jdrop2 БП;
code СЧ  ↔ КП3 ↔ КСЧ  next;
code ЗН  КЗН  next;
code [x]  К[x]  next;
code {x}  К{x}  next;
code |x|  К|x|  next;
code /-/  /-/  next;

code radians  1  p: SetAng  9045 П  Fo  next;
code degrees  0  SetAng БП;
code arcsin  Farcsin  next;
code arccos  Farccos  next;
code arctg  Farctg  next;
code sin  Fsin  next;
code cos  Fcos  next;
code tg  Ftg  next;

code key  П8 ↔ КП3  ChGet ПП  ИП8 ↔  next;
code emit  ↔ П8 ↔ ChPut ПП  ИП8  jdrop БП;
code bye  С/П  next;         \ Вообще-то после перезапуска ещё надо клавиатуру инициализировать init

code test                    \ === Примитив test ===
  begin
    ChGet ПП ПС
    ChPut ПП КГРФ
    ИПС 2 7 - x=0
  until
next;

:: testAAA                   \ Определение высокого уровня, проверка работоспособности
     test
     exit
;;

\ ------------------------------------------------------
\ Каллисто-2: Инициализация десятичных регистров МК-161.
\ ------------------------------------------------------

NEXT 09 d!                   \ Можно ещё выделить регистры для "других" NEXT
bc [1-] &Autoexec &!         \ Адрес автозапуска шитого кода (ссылка вперёд)
&KPNum ptrKbdInt d!          \ Клавиатура по умолчанию в режиме Num

\ -------------------------------------
\ Каллисто-2: Байтовые регистры МК-161.
\ -------------------------------------

\ === Автозапускающийся шитый код ===
init                         \ Начальная инициализация
lit8 62 b, emit              \ Вывод приглашения ">"
testAAA                      \ Примитивный диалог
bye                          \ Выход в калькулятор

.( Compiled: PC=) pc .  .( BC=) bc .

FRCHK

\ Запишем Каллисто-2 на диск
save.mkp save.mkb save.mkd

.( Saved!) CR

BYE
\EOF
